require 'bcrypt'
require 'grape-entity'

# additional requires at bottom

module Subledger
  module Domain
    class ActivatableError                < Error; end
    class ArchivableError                 < Error; end
    class AttributableError               < Error; end
    class CollectableError                < Error; end
    class CreatableError                  < Error; end
    class DescribableError                < Error; end
    class DescribableReportRenderingError < Error; end
    class IdentifiableError               < Error; end
    class PostableError                   < Error; end
    class ProgressableError               < Error; end
    class ReadableError                   < Error; end
    class RestableError                   < Error; end
    class StorableError                   < Error; end
    class TimeableError                   < Error; end
    class UpdatableError                  < Error; end
    class VersionableError                < Error; end

    def self.included base
      base.extend DomainClass
    end

    module DomainClass
      def keys
        self::Entity.exposures.keys
      end

      def entity_name
        self::Entity.instance_variable_get( :@root ).to_sym
      end

      def collection_name
        self::Entity.instance_variable_get( :@collection_root ).to_sym
      end

      def raise_unless_creatable args
        raise NoMethodError, 'subclasses must provide self.raise_unless_creatable'
      end
    end

    def == other
      if respond_to?( :id )       and
         other.respond_to?( :id ) and
         ( ( not id.nil? ) and ( not other.id.nil? ) )
        id == other.id
      else
        super
      end
    end

    def collection_name
      self.class.collection_name
    end

    def entity_name
      self.class.entity_name
    end

    def to_s
      "#{entity_name}: #{ MultiJson.dump serializable_hash }"
    end

    alias :inspect :to_s

    private

    def short_klass_name
      self.class.name.match( /[^:]+$/ )[0] + ': '
    end
  end
end

require 'subledger/domain/formatters'
require 'subledger/domain/roles'

require 'subledger/domain/value'

require 'subledger/domain/account'
require 'subledger/domain/balance'
require 'subledger/domain/book'
require 'subledger/domain/category'
require 'subledger/domain/control'
require 'subledger/domain/identity'
require 'subledger/domain/journal_entry'
require 'subledger/domain/key'
require 'subledger/domain/line'
require 'subledger/domain/org'
require 'subledger/domain/report'
require 'subledger/domain/report_rendering'
