module Subledger
  module Domain
    class BookError < Error; end

    class Book

      include Domain

      include Roles::Attributable
      include Roles::Describable
      include Roles::Identifiable
      include Roles::Storable
      include Roles::Versionable

      include Roles::Creatable
      include Roles::Readable
      include Roles::Updatable

      include Roles::Collectable

      include Roles::Activatable
      include Roles::Archivable

      include Roles::Restable

      attr_reader :org

      def self.post_keys
        [ :org, :description, :reference ]
      end

      def self.patch_keys
        # TODO should :id be in patch_keys?

        [ :id, :description, :reference, :version ]
      end

      def self.root_klass
        Book
      end

      def self.sub_klasses
        [ active_klass, archived_klass ]
      end

      def self.active_klass
        ActiveBook
      end

      def self.archived_klass
        ArchivedBook
      end

      def initialize args
        describable args
        identifiable args
        storable args
        versionable args

        @org  = args[:org]
      end

      class Entity < Grape::Entity
        root 'books', 'book'

        expose :id,          :documentation => { :type => 'string',  :desc => 'book ID' }

        expose :org,         :format_with   => :id,
                             :documentation => { :type => 'string',  :desc => 'org ID' }

        expose :description, :documentation => { :type => 'string',  :desc => 'description' }

        expose :reference,   :documentation => { :type => 'string',  :desc => 'reference URI' }

        expose :version,     :documentation => { :type => 'integer', :desc => 'version' }
      end

      private

      def self.raise_unless_creatable args
        org = args[:org]

        if org.nil? or not org.kind_of? Org
          raise BookError, ':org is required and must be an Org'
        elsif UUID.invalid? org.id
          raise BookError, ':org must have a valid :id'
        end
      end
    end

    class ArchivedBook < Book
      class Entity < Book::Entity
        root 'archived_books', 'archived_book'
      end

      def self.sub_klasses
        [ archived_klass ]
      end
    end

    class ActiveBook < Book
      class Entity < Book::Entity
        root 'active_books', 'active_book'
      end

      def self.sub_klasses
        [ active_klass ]
      end
    end
  end
end
