module Subledger
  module Domain
    class KeyError < Error; end

    class Key

      include Domain

      include Roles::Attributable
      include Roles::Identifiable
      include Roles::Storable

      include Roles::Creatable
      include Roles::Readable
      include Roles::Collectable

      include Roles::Activatable
      include Roles::Archivable

      include Roles::Restable

      attr_reader :identity, :secret

      def self.post_keys
        [ :id, :identity ]
      end

      def self.root_klass
        Key
      end

      def self.sub_klasses
        [ archived_klass, active_klass ]
      end

      def self.active_klass
        ActiveKey
      end

      def self.archived_klass
        ArchivedKey
      end

      def initialize args
        identifiable args
        storable args

        @identity = args[:identity]

        # TODO separate new and create

        if args[:bcrypt].nil?
          @secret = args[:secret] || UUID.as_string

          @bcrypt = BCrypt::Password.create( @secret, :cost => 5 )
        elsif args[:bcrypt]
          @secret = args[:secret]

          @bcrypt = BCrypt::Password.new args[:bcrypt]
        end
      end

      def authenticates? this_secret
        bcrypt == this_secret
      end

      class Entity < Grape::Entity
        root 'keys', 'key'

        expose :id,       :documentation => { :type => 'string',  :desc => 'key ID' }

        expose :identity, :format_with => :id,
                          :documentation => { :type => 'string',  :desc => 'identity ID' }

        expose :secret,   :documentation => { :type => 'string',  :desc => 'key secret' }
      end

      private

      attr_reader :bcrypt

      def self.raise_unless_creatable args
        identity = args[:identity]

        if identity.nil? or not identity.kind_of? Identity
          raise KeyError, ':identity is required and must be an Identity'
        elsif UUID.invalid? identity.id
          raise KeyError, ':identity must have a valid :id'
        end
      end
    end

    class ArchivedKey < Key
      class Entity < Key::Entity
        root 'archived_keys', 'archived_key'
      end

      def self.sub_klasses
        [ archived_klass ]
      end
    end

    class ActiveKey < Key
      class Entity < Key::Entity
        root 'active_keys', 'active_key'
      end

      def self.sub_klasses
        [ active_klass ]
      end
    end
  end
end
