module Subledger
  module Domain
    class OrgError < Error; end

    class Org

      include Domain

      include Roles::Attributable
      include Roles::Describable
      include Roles::Identifiable
      include Roles::Storable
      include Roles::Versionable

      include Roles::Creatable
      include Roles::Readable
      include Roles::Updatable

      include Roles::Activatable
      include Roles::Archivable

      include Roles::Restable

      attr_accessor :bucket_name

      def self.root_klass
        Org
      end

      def self.post_keys
        [ :description, :reference, :bucket_name ]
      end

      def self.patch_keys
        [ :id, :description, :reference, :bucket_name, :version ]
      end

      def self.sub_klasses
        [ active_klass, archived_klass ]
      end

      def self.active_klass
        ActiveOrg
      end

      def self.archived_klass
        ArchivedOrg
      end

      def self.create args
        org = super

        args[:store].add_initial_controls_for org

        org
      end

      def initialize args
        describable args
        identifiable args
        storable args
        versionable args

        @bucket_name = args[:bucket_name]
      end

      class Entity < Grape::Entity
        root 'orgs', 'org'

        expose :id,          :documentation => { :type => 'string',  :desc => 'org ID' }

        expose :description, :documentation => { :type => 'string',  :desc => 'description' }

        expose :reference,   :documentation => { :type => 'string',  :desc => 'reference URI' }

        expose :bucket_name, :documentation => { :type => 'string',  :desc => 'backup URI' }

        expose :version,     :documentation => { :type => 'integer', :desc => 'version' }
      end

      private

      def self.raise_unless_creatable args
        store = args[:store]

        store.raise_unless_bucket_name_valid args
      end
    end

    class ArchivedOrg < Org
      class Entity < Org::Entity
        root 'archived_orgs', 'archived_org'
      end

      def self.sub_klasses
        [ archived_klass ]
      end
    end

    class ActiveOrg < Org
      class Entity < Org::Entity
        root 'active_orgs', 'active_org'
      end

      def self.sub_klasses
        [ active_klass ]
      end
    end
  end
end
