module Subledger
  module Domain
    module Roles
      module Creatable
        def self.included base
          base.extend CreatableClass
        end

        module CreatableClass
          def create args
            validate_creatability args

            creatable = args[:client].send active_klass.collection_name, args

            args[:store].create creatable
          end

          def new_or_create args
            id = args[:id]

            if id.nil?
              item = active_klass.create args

              yield item

              item
            else
              active_klass.new args
            end
          end

          private

          def validate_creatability args
            raise_unless_creatable args
            validate_creatability_modules args
          end

          def validate_creatability_modules args
            self.included_modules.each do |mod|
              if mod.respond_to? :raise_unless_creatable
                mod.raise_unless_creatable args
              end
            end
          end
        end

        def create
          created = self.class.create self.attributes

          initialize created.attributes

          self
        end
      end
    end
  end
end
