module Subledger
  module Domain
    module Roles
      module Restable
        def post_hash
          hash = { }

          self.class.post_keys.each do |key|
            value = self.send key

            hash[key] = translate( key, value ) unless exclude? value
          end

          hash
        end

        def patch_hash
          hash = { }

          self.class.patch_keys.each do |key|
            value = self.send key

            hash[key] = translate( key, value ) unless exclude? value
          end

          hash
        end

        def serializable_hash
          entity = self.class::Entity.new self

          hash = entity.serializable_hash

          hash.each do |key, value|
            hash.delete( key ) if exclude? value
          end

          hash
        end

        def to_json
          MultiJson.dump serializable_hash
        end

        private

        # TODO elimate this using serializable hash
        def translate key, value
          case key.to_s
            when /at$/
              value.iso8601(3)
            when 'org', 'book', 'account', 'journal_entry', 'identity', 'report'
              value.id
            when 'value'
              value.rest_hash
            when 'normal_balance'
              value.type
            else
              value
          end
        end

        def exclude? value
          value.nil? or ( value.kind_of?( String ) and value.length.zero? )
        end
      end
    end
  end
end
