# additional requires at bottom

module Subledger
  module Domain
    class ValueError < Error; end

    module Value

      def self.included base
        base.extend ValueClass
      end

      module ValueClass
        def raise_unless_creatable args
          value = args[:value]

          if value.nil? or not value.kind_of? Value
            raise ValueError, ':value is required and must be a Value'
          end
        end
      end

      attr_reader :amount

      def initialize arg_amount=0
        if arg_amount.nil?
          raise ValueError, 'amount may not be nil'
        end

        validate_amount(arg_amount) if arg_amount.respond_to?(:match)

        big_amount = BigDecimal arg_amount

        validate_sign big_amount

        validate_amount( amount_to_s big_amount )

        @amount = big_amount
      end

      def + other
        (Balance.new + value + other.value).value
      end

      def == other
        self.value.class == other.value.class and amount == other.amount
      end

      def value
        self
      end

      def to_s
        "#{self.class.type}: #{amount_to_s @amount}"
      end

      alias :inspect :to_s

      private

      def amount_to_s arg_amount
        if arg_amount.zero?
          '0'
        else
          arg_amount.to_s( 'F' ).chomp( '.0' ).sub( /^0+/, '' )
        end
      end

      def validate_sign arg_amount
        unless arg_amount > 0
          raise ValueError, 'amount must be positive'
        end
      end

      ERROR_TEXT = 'amount is limited to 15 whole digits and 12 fractional digits'

      def validate_amount arg_amount
        unless arg_amount.match /^\d*[.]?\d*$/
          raise ValueError, "amount (#{arg_amount}) must be numeric"
        end

        return if BigDecimal(arg_amount).zero?

        dupped_arg_amount = arg_amount.dup

        dupped_arg_amount.sub! /^0+/, ''
        dupped_arg_amount.sub! /[.](\d)*0+$/, '.\\1'

        if dupped_arg_amount.include? '.'
          whole, fraction = dupped_arg_amount.split '.'

          whole    ||= ''
          fraction ||= ''

          if whole.length > 15 or fraction.length > 12
            raise ValueError, ERROR_TEXT
          end
        else
          if dupped_arg_amount.length > 15
            raise ValueError, ERROR_TEXT
          end
        end

      end

    end
  end
end

require 'subledger/domain/value/credit'
require 'subledger/domain/value/debit'
require 'subledger/domain/value/zero'
