module Subledger
  module Interface
    class ClientError < Error; end

    class Client

      private

      attr_reader :client_args

      COLLECTIONS = { :identity                    => { :klass           => Domain::Identity,
                                                        :root_collection => :identities,
                                                        :state           => :active },
                      :identities                  => { :klass           => Domain::Identity,
                                                        :root_collection => :identities,
                                                        :state           => :active },
                      :archived_identity           => { :klass           => Domain::ArchivedIdentity,
                                                        :root_collection => :identities,
                                                        :state           => :archived },
                      :archived_identities         => { :klass           => Domain::ArchivedIdentity,
                                                        :root_collection => :identities,
                                                        :state           => :archived },
                      :active_identity             => { :klass           => Domain::ActiveIdentity,
                                                        :root_collection => :identities,
                                                        :state           => :active },
                      :active_identities           => { :klass           => Domain::ActiveIdentity,
                                                        :root_collection => :identities,
                                                        :state           => :active },
                      :key                         => { :klass           => Domain::Key,
                                                        :root_collection => :keys,
                                                        :state           => :active },
                      :keys                        => { :klass           => Domain::Key,
                                                        :root_collection => :keys,
                                                        :state           => :active },
                      :archived_key                => { :klass           => Domain::ArchivedKey,
                                                        :root_collection => :keys,
                                                        :state           => :archived },
                      :archived_keys               => { :klass           => Domain::ArchivedKey,
                                                        :root_collection => :keys,
                                                        :state           => :archived },
                      :active_key                  => { :klass           => Domain::ActiveKey,
                                                        :root_collection => :keys,
                                                        :state           => :active },
                      :active_keys                 => { :klass           => Domain::ActiveKey,
                                                        :root_collection => :keys,
                                                        :state           => :active },
                      :control                     => { :klass           => Domain::Control,
                                                        :root_collection => :controls,
                                                        :state           => :active },
                      :controls                    => { :klass           => Domain::Control,
                                                        :root_collection => :controls,
                                                        :state           => :active },
                      :archived_control            => { :klass           => Domain::ArchivedControl,
                                                        :root_collection => :controls,
                                                        :state           => :archived },
                      :archived_controls           => { :klass           => Domain::ArchivedControl,
                                                        :root_collection => :controls,
                                                        :state           => :archived },
                      :active_control              => { :klass           => Domain::ActiveControl,
                                                        :root_collection => :controls,
                                                        :state           => :active },
                      :active_controls             => { :klass           => Domain::ActiveControl,
                                                        :root_collection => :controls,
                                                        :state           => :active },
                      :org                         => { :klass           => Domain::Org,
                                                        :root_collection => :orgs,
                                                        :state           => :active },
                      :orgs                        => { :klass           => Domain::Org,
                                                        :root_collection => :orgs,
                                                        :state           => :active },
                      :archived_org                => { :klass           => Domain::ArchivedOrg,
                                                        :root_collection => :orgs,
                                                        :state           => :archived },
                      :archived_orgs               => { :klass           => Domain::ArchivedOrg,
                                                        :root_collection => :orgs,
                                                        :state           => :archived },
                      :active_org                  => { :klass           => Domain::ActiveOrg,
                                                        :root_collection => :orgs,
                                                        :state           => :active },
                      :active_orgs                 => { :klass           => Domain::ActiveOrg,
                                                        :root_collection => :orgs,
                                                        :state           => :active },
                      :book                        => { :klass           => Domain::Book,
                                                        :root_collection => :books,
                                                        :state           => :active },
                      :books                       => { :klass           => Domain::Book,
                                                        :root_collection => :books,
                                                        :state           => :active },
                      :archived_book               => { :klass           => Domain::ArchivedBook,
                                                        :root_collection => :books,
                                                        :state           => :archived },
                      :archived_books              => { :klass           => Domain::ArchivedBook,
                                                        :root_collection => :books,
                                                        :state           => :archived },
                      :active_book                 => { :klass           => Domain::ActiveBook,
                                                        :root_collection => :books,
                                                        :state           => :active },
                      :active_books                => { :klass           => Domain::ActiveBook,
                                                        :root_collection => :books,
                                                        :state           => :active },
                      :account                     => { :klass           => Domain::Account,
                                                        :root_collection => :accounts,
                                                        :state           => :active },
                      :accounts                    => { :klass           => Domain::Account,
                                                        :root_collection => :accounts,
                                                        :state           => :active },
                      :archived_account            => { :klass           => Domain::ArchivedAccount,
                                                        :root_collection => :accounts,
                                                        :state           => :archived },
                      :archived_accounts           => { :klass           => Domain::ArchivedAccount,
                                                        :root_collection => :accounts,
                                                        :state           => :archived },
                      :active_account              => { :klass           => Domain::ActiveAccount,
                                                        :root_collection => :accounts,
                                                        :state           => :active },
                      :active_accounts             => { :klass           => Domain::ActiveAccount,
                                                        :root_collection => :accounts,
                                                        :state           => :active },
                      :category_accounts           => { :klass           => Domain::Account,
                                                        :root_collection => :category_accounts,
                                                        :state           => :nil },
                      :journal_entry               => { :klass           => Domain::JournalEntry,
                                                        :root_collection => :journal_entries,
                                                        :state           => :active },
                      :journal_entries             => { :klass           => Domain::JournalEntry,
                                                        :root_collection => :journal_entries,
                                                        :state           => :active },
                      :archived_journal_entry      => { :klass           => Domain::ArchivedJournalEntry,
                                                        :root_collection => :journal_entries,
                                                        :state           => :archived },
                      :archived_journal_entries    => { :klass           => Domain::ArchivedJournalEntry,
                                                        :root_collection => :journal_entries,
                                                        :state           => :archived },
                      :active_journal_entry        => { :klass           => Domain::ActiveJournalEntry,
                                                        :root_collection => :journal_entries,
                                                        :state           => :active },
                      :active_journal_entries      => { :klass           => Domain::ActiveJournalEntry,
                                                        :root_collection => :journal_entries,
                                                        :state           => :active },
                      :posting_journal_entry       => { :klass           => Domain::PostingJournalEntry,
                                                        :root_collection => :journal_entries,
                                                        :state           => :posting },
                      :posting_journal_entries     => { :klass           => Domain::PostingJournalEntry,
                                                        :root_collection => :journal_entries,
                                                        :state           => :posting },
                      :posted_journal_entry        => { :klass           => Domain::PostedJournalEntry,
                                                        :root_collection => :journal_entries,
                                                        :state           => :posted },
                      :posted_journal_entries      => { :klass           => Domain::PostedJournalEntry,
                                                        :root_collection => :journal_entries,
                                                        :state           => :posted },
                      :line                        => { :klass           => Domain::Line,
                                                        :root_collection => :lines,
                                                        :state           => :active },
                      :lines                       => { :klass           => Domain::Line,
                                                        :root_collection => :lines,
                                                        :state           => :active },
                      :archived_line               => { :klass           => Domain::ArchivedLine,
                                                        :root_collection => :lines,
                                                        :state           => :archived },
                      :archived_lines              => { :klass           => Domain::ArchivedLine,
                                                        :root_collection => :lines,
                                                        :state           => :archived },
                      :active_line                 => { :klass           => Domain::ActiveLine,
                                                        :root_collection => :lines,
                                                        :state           => :active },
                      :active_lines                => { :klass           => Domain::ActiveLine,
                                                        :root_collection => :lines,
                                                        :state           => :active },
                      :posted_line                 => { :klass           => Domain::PostedLine,
                                                        :root_collection => :lines,
                                                        :state           => :posted },
                      :posted_lines                => { :klass           => Domain::PostedLine,
                                                        :root_collection => :lines,
                                                        :state           => :posted },
                      :account_line                => { :klass           => Domain::PostedLine,
                                                        :root_collection => :account_lines,
                                                        :state           => :nil },
                      :account_lines               => { :klass           => Domain::PostedLine,
                                                        :root_collection => :account_lines,
                                                        :state           => :nil },
                      :category                    => { :klass           => Domain::Category,
                                                        :root_collection => :categories,
                                                        :state           => :active },
                      :categories                  => { :klass           => Domain::Category,
                                                        :root_collection => :categories,
                                                        :state           => :active },
                      :archived_category           => { :klass           => Domain::ArchivedCategory,
                                                        :root_collection => :categories,
                                                        :state           => :archived },
                      :archived_categories         => { :klass           => Domain::ArchivedCategory,
                                                        :root_collection => :categories,
                                                        :state           => :archived },
                      :active_category             => { :klass           => Domain::ActiveCategory,
                                                        :root_collection => :categories,
                                                        :state           => :active },
                      :active_categories           => { :klass           => Domain::ActiveCategory,
                                                        :root_collection => :categories,
                                                        :state           => :active },
                      :report_categories           => { :klass           => Domain::Category,
                                                        :root_collection => :report_categories,
                                                        :state           => :nil },
                      :report                      => { :klass           => Domain::Report,
                                                        :root_collection => :reports,
                                                        :state           => :active },
                      :reports                     => { :klass           => Domain::Report,
                                                        :root_collection => :reports,
                                                        :state           => :active },
                      :archived_report             => { :klass           => Domain::ArchivedReport,
                                                        :root_collection => :reports,
                                                        :state           => :archived },
                      :archived_reports            => { :klass           => Domain::ArchivedReport,
                                                        :root_collection => :reports,
                                                        :state           => :archived },
                      :active_report               => { :klass           => Domain::ActiveReport,
                                                        :root_collection => :reports,
                                                        :state           => :active },
                      :active_reports              => { :klass           => Domain::ActiveReport,
                                                        :root_collection => :reports,
                                                        :state           => :active },
                      :report_rendering            => { :klass           => Domain::ReportRendering,
                                                        :root_collection => :report_renderings,
                                                        :state           => :building },
                      :report_renderings           => { :klass           => Domain::ReportRendering,
                                                        :root_collection => :report_renderings,
                                                        :state           => :building },
                      :building_report_rendering   => { :klass           => Domain::BuildingReportRendering,
                                                        :root_collection => :report_renderings,
                                                        :state           => :building },
                      :building_report_renderings  => { :klass           => Domain::BuildingReportRendering,
                                                        :root_collection => :report_renderings,
                                                        :state           => :building },
                      :completed_report_rendering  => { :klass           => Domain::CompletedReportRendering,
                                                        :root_collection => :report_renderings,
                                                        :state           => :completed },
                      :completed_report_renderings => { :klass           => Domain::CompletedReportRendering,
                                                        :root_collection => :report_renderings,
                                                        :state           => :completed }
      }

      public

      attr_reader :store

      def initialize args
        @client_args = args

        # TODO client should not have @store and #store

        @store = args[:store]
      end

      COLLECTIONS.each do |name, attributes|
        klass           = attributes[:klass]
        state           = attributes[:state] == :nil ? 'nil' : attributes[:state].inspect
        root_collection = attributes[:root_collection]

        method_text = <<-"EOM"
          def #{name} *args
            @#{name}_dispatcher ||= Dispatcher.new @client_args.
                                                     merge(
                                                       { :klass           => #{klass},
                                                         :client          => self,
                                                         :state           => #{state},
                                                         :root_collection => :#{root_collection} } )

            if args.empty?
              @#{name}_dispatcher
            else
              @#{name}_dispatcher.new args.first.dup
            end
          end
        EOM

        class_eval method_text
      end

      def debit *args
        args.empty? ? Domain::Debit  : Domain::Debit.new( *args )
      end

      def credit *args
        args.empty? ? Domain::Credit : Domain::Credit.new( *args )
      end

      def zero *args
        Domain::Zero.new
      end

      def balance *args
        Domain::Balance.new *args
      end
    end
  end
end
