module Subledger
  module Path
    def self.for_collection args
      "/#{ API_VERSION }#{ path args }"
    end

    def self.for_entity args
      "#{ for_collection args }/#{ args[ :anchor ].id }"
    end

    def self.for_entity_without_api_version args
      "#{ path args }/#{ args[ :anchor ].id }"
    end

    def self.for_collect args
      action          = args[:action]
      state           = args[:state]
      limit           = args[:limit]
      collection_name = args[:collection_name]

      key   = key_for args
      value = URI.escape value_for( args ).to_s

      path = for_collection( args ) + '?'

      query_params = []

      query_params << "state=#{    state  }" unless collection_name == :account_lines
      query_params << "action=#{   action }"
      query_params << "#{ key }=#{ value  }" unless value.respond_to?( :length ) and value.length.zero?
      query_params << "limit=#{    limit  }" unless limit.nil?

      path + query_params.join('&')
    end

    private

    COLLECTION_PATHS = {
                         :identities        => lambda { |anchor| 'identities' },
                         :keys              => lambda { |anchor| "identities/#{anchor.identity.id}/keys" },
                         :controls          => lambda { |anchor| "identities/#{anchor.identity.id}/controls" },
                         :orgs              => lambda { |anchor| 'orgs' },
                         :books             => lambda { |anchor| "orgs/#{anchor.org.id}/books" },
                         :accounts          => lambda { |anchor| "orgs/#{anchor.book.org.id}/books/#{anchor.book.id}/accounts" },
                         :journal_entries   => lambda { |anchor| "orgs/#{anchor.book.org.id}/books/#{anchor.book.id}/journal_entries" },
                         :lines             => lambda { |anchor| "orgs/#{anchor.journal_entry.book.org.id}/books/#{anchor.journal_entry.book.id}/journal_entries/#{anchor.journal_entry.id}/lines" },
                         :account_lines     => lambda { |anchor| "orgs/#{anchor.account.book.org.id}/books/#{anchor.account.book.id}/accounts/#{anchor.account.id}/lines" },
                         :reports           => lambda { |anchor| "orgs/#{anchor.book.org.id}/books/#{anchor.book.id}/reports" },
                         :categories        => lambda { |anchor| "orgs/#{anchor.book.org.id}/books/#{anchor.book.id}/categories" },
                         :report_renderings => lambda { |anchor| "orgs/#{anchor.book.org.id}/books/#{anchor.book.id}/report_renderings" }
                       }

    def self.path args
      anchor = args[:anchor]

      collection_name = CollectionName.without_state args

      path = COLLECTION_PATHS[collection_name].call anchor

      "/#{path}"
    end

    KEY_FOR = { :books             => 'description',
                :accounts          => 'description',
                :journal_entries   => 'effective_at',
                :lines             => 'order',
                :account_lines     => 'effective_at',
                :reports           => 'description',
                :categories        => 'description',
                :report_renderings => 'rendered_at'
              }

    def self.key_for args
      action = args[:action]

      if [ :preceding, :following ].include? action
        'id'
      else
        KEY_FOR[ CollectionName.without_state args ]
      end
    end

    VALUE_FOR = { :books             => lambda { |anchor| anchor.description },
                  :accounts          => lambda { |anchor| anchor.description },
                  :journal_entries   => lambda { |anchor| anchor.effective_at.iso8601(3) },
                  :lines             => lambda { |anchor| anchor.order },
                  :account_lines     => lambda { |anchor| anchor.effective_at.iso8601(3) },
                  :reports           => lambda { |anchor| anchor.description },
                  :categories        => lambda { |anchor| anchor.description },
                  :report_renderings => lambda { |anchor| anchor.rendered_at.iso8601(3) }
                }

    def self.value_for args
      action = args[:action]
      anchor = args[:anchor]

      if [ :preceding, :following ].include? action
        anchor.id
      else
        collection_name = CollectionName.without_state args

        VALUE_FOR[collection_name].call anchor
      end
    end
  end
end
