module Subledger
  module Rest
    def self.to_args rest_args, client
      args = { }

      rest_args.each do |key, value|
        key_symbol = key.to_sym

        unless value.nil?
          code = LAMBDAS[key]

          args[key_symbol] = if code.nil? or value.kind_of? Domain
                               value
                             else
                               code.call client, args, key, value
                             end
        end
      end

      args
    end

    def self.to_balance rest_args, client
      to_args( rest_args, client )[:balance]
    end

    def self.present_entity entity
      { entity.entity_name => entity.serializable_hash }
    end

    def self.present_collection entity_name, collection
      entities = collection.collect { |entity| entity.serializable_hash }

      { entity_name => entities }
    end

    private

    ACCOUNT = lambda do |client, args, key, value|
                client.account :id => value
              end

    # TODO eliminate if condition in BALANCE LAMBDA

    BALANCE = lambda do |client, args, key, value|
                if value.has_key? :debit_value
                  client.balance :debit_value  => debit_value(  client, value[:debit_value ]['amount'] ),
                                 :credit_value => credit_value( client, value[:credit_value]['amount'] )
                else
                  client.balance :debit_value  => debit_value(  client, value['debit_value' ]['amount'] ),
                                 :credit_value => credit_value( client, value['credit_value']['amount'] )
                end
              end

    BOOK = lambda do |client, args, key, value|
             client.book :id => value
            end

    CATEGORIES = lambda do |client, args, key, value|
                   args[:categories] = MultiJson.dump value
                 end

    IDENTITY = lambda do |client, args, key, value|
                 client.identity :id => value
               end

    JOURNAL_ENTRY = lambda do |client, args, key, value|
                      client.journal_entry :id => value
                    end

    MODE = lambda do |client, args, key, value|
             value.to_sym
           end

    NB   = lambda do | client, args, key, value |
             client.send value
           end

    ORG = lambda do |client, args, key, value|
            client.org :id => value
          end

    TIME = lambda do |client, args, key, value|
             Subledger.parse_time key, value
           end

    VALUE = lambda do |client, args, key, value|
              client.send value['type'], value['amount']
            end

    VERSION = lambda do |client, args, key, value|
                value.to_i
              end

    REPORT = lambda do |client, args, key, value|
               client.report :id => value
             end

    LAMBDAS = {
      'account'        => ACCOUNT,
      'balance'        => BALANCE,
      'book'           => BOOK,
      'categories'     => CATEGORIES,
      'completed_at'   => TIME,
      'credit_value'   => VALUE,
      'debit_value'    => VALUE,
      'effective_at'   => TIME,
      'identity'       => IDENTITY,
      'journal_entry'  => JOURNAL_ENTRY,
      'mode'           => MODE,
      'normal_balance' => NB,
      'org'            => ORG,
      'posted_at'      => TIME,
      'rendered_at'    => TIME,
      'report'         => REPORT,
      'value'          => VALUE,
      'version'        => VERSION
    }

    def self.debit_value client, amount
      BigDecimal( amount ) == 0 ? client.zero : client.debit( amount )
    end

    def self.credit_value client, amount
      BigDecimal( amount ) == 0 ? client.zero : client.credit( amount )
    end
  end
end
