module Subledger
  module Store
    module Api
      module Errors
        # Raised when the Subledger API returns a 400 status code
        class BadRequest < Error; end

        # Raised when the Subledger API returns a 401 status code
        class Unauthorized < Error; end

        # Raised when the Subledger API returns a 403 status code
        class Forbidden < Error; end

        # Raised when the Subledger API returns a 404 status code
        class NotFound < Error; end

        # Raised when the Subledger API returns a 406 status code
        class NotAcceptable < Error; end

        # Raised when the Subledger API returns a 409 status code
        class Conflict < Error; end

        # Raised when the Subledger API returns a 422 status code
        class UnprocessableEntity < Error; end

        # Raised when the Subledger API returns a 500 status code
        class InternalServerError < Error; end

        # Raised when the Subledger API returns a 501 status code
        class NotImplemented < Error; end

        # Raised when the Subledger API returns a 502 status code
        class BadGateway < Error; end

        # Raised when the Subledger API returns a 503 status code
        class ServiceUnavailable < Error; end

        # Raise when the Subledger API returns any other status code
        class HttpError < Error; end

        class ServerNotAvailable < Error
          def initialize wrapped
            @wrapped = wrapped
          end

          def message
            "#{super} was not avaiable. Please check your configuration. (#{@wrapped.message})"
          end
        end

        class ResponseError < Faraday::Response::Middleware

          EXCEPTIONS = {
            400 => BadRequest,
            401 => Unauthorized,
            403 => Forbidden,
            404 => NotFound,
            406 => NotAcceptable,
            409 => Store::UpdateConflictError,
            422 => UnprocessableEntity,
            500 => InternalServerError,
            501 => NotImplemented,
            502 => BadGateway,
            503 => ServiceUnavailable
          }

          def call env
            @app.call(env).on_complete do
              status = env[:status].to_i

              if ( 400..600 ).include? status
                raise exception_for( status ), response_for( env )
              end
            end
          end

          def exception_for status
            EXCEPTIONS[status] || HttpError
          end

          def response_for env
            { :status  => env[:status],
              :url     => url_for( env ),
              :headers => env[:response_headers],
              :body    => env[:body] }
          end

          def url_for env
            env[:url].to_s
          end
        end
      end
    end
  end
end
