module Subledger
  module Store
    module Api
      class Store

        include Subledger::Store

        include Balance
        include Categories
        include Collect
        include Create
        include CreateAndPost
        include CreateLine
        include FirstAndLastLine
        include Post
        include Progress
        include Read
        include Reports
        include Update
        include Activate
        include Archive
        include CreateIdentity

        def add_initial_controls_for org
          # Server handles this functionality
        end

        def add_initial_controls_to identity
          # Server handles this functionality
        end

        def raise_unless_bucket_name_valid args
          # No endpoints for this
        end

        def create_backup anchor
          # No endpoints for this
        end

        def backup_exists? anchor
          # No endpoints for this
        end

        private

        attr_reader :http

        def base_url
          "#{SCHEME}#{DOMAIN}"
        end

        def setup
          @http = Faraday.new :url => base_url do |conn|
                    unless auth_key.nil?
                      conn.basic_auth auth_key.id, auth_key.secret
                    end

                    conn.request    :json
                    conn.response   :json, :content_type => /^json$/
                    conn.use        Subledger::Store::Api::Errors::ResponseError
                    conn.adapter    Faraday.default_adapter
                  end
        end

        def parse_json json_body
          MultiJson.load json_body
        end

        def klass_method_from parsed_json
          parsed_json.keys.first
        end

        def new_or_initialize json_body, initializable
          client = initializable.client

          parsed_json = parse_json json_body

          klass_method = klass_method_from parsed_json

          response_hash = parsed_json[klass_method]

          args = Rest.to_args response_hash, client

          args.merge! :json => MultiJson.dump( response_hash )

          if initializable.respond_to? :post_delay
            args.merge! :post_delay => initializable.post_delay
          end

          new_item = client.send klass_method, args

          if initializable.entity_name == klass_method.to_sym
            initializable.send :initialize, new_item.attributes
          else
            initializable = new_item
          end

          initializable
        end
      end
    end
  end
end
