require 'securerandom'

module Subledger
  module UUID

    SPACE   = 62 ** 22

    CHARS = ('0'..'9').to_a + ('a'..'z').to_a + ('A'..'Z').to_a

    VALID = /^[0-9A-Za-z]{22}$/

    def self.valid? uuid
      !uuid.nil? and uuid =~ VALID
    end

    def self.invalid? uuid
      uuid.nil? or uuid !~ VALID
    end

    def self.as_string
      to_string as_bignum
    end

    def self.as_bignum
      SecureRandom.random_number SPACE
    end

    def self.to_string bignum
      base_62 = ''

      22.times do
        base_62 << CHARS[ bignum.modulo 62 ]
        bignum /= 62
      end

      base_62.reverse
    end

    def self.to_bignum uuid
      bignum = 0

      power = 0

      uuid.split('').reverse.each do |character|
        bignum += CHARS.index( character ) * ( 62 ** power )
        power += 1
      end

      bignum
    end
  end
end
